import { IEventProps, Event } from '@designable/shared';
import { Engine, ITreeNode, ScreenType, Shortcut, Viewport, Workbench, Workspace, TreeNode, JSONSchema7 } from './models';
export declare type IEngineProps<T = Event> = IEventProps<T> & {
    sourceIdAttrName?: string;
    nodeIdAttrName?: string;
    outlineNodeIdAttrName?: string;
    nodeHelpersIdAttrName?: string;
    defaultComponentTree?: ITreeNode[];
    defaultScreenType?: ScreenType;
    shortcuts?: Shortcut[];
};
export declare type IEngineContext = {
    workspace: Workspace;
    workbench: Workbench;
    engine: Engine;
    viewport: Viewport;
};
export declare type IControlType = boolean | ((node: TreeNode) => boolean);
export declare type IControlNodeMetaType = {
    componentName: string;
    id?: string;
    maxInstances?: number;
    minInstances?: number;
};
export declare type IControlNodeType = string | IControlNodeMetaType | ((node: TreeNode) => IControlNodeMetaType);
export interface IDesignerControllerProps {
    package?: string;
    registry?: string;
    version?: string;
    path?: string;
    title?: string;
    description?: string;
    icon?: string;
    group?: string;
    droppable?: IControlType;
    draggable?: IControlType;
    deletable?: IControlType;
    cloneable?: IControlType;
    resizable?: IControlType;
    inlineLayout?: IControlType;
    inlineChildrenLayout?: IControlType;
    selfRenderChildren?: IControlType;
    propsSchema?: JSONSchema7;
    effects?: (engine: Engine) => void;
    getDragNodes?: (node: TreeNode) => TreeNode[];
    getComponentProps?: (node: TreeNode) => any;
    allowAppend?: (target: TreeNode, sources?: TreeNode[]) => boolean;
    allowSiblings?: (target: TreeNode, sources?: TreeNode[]) => boolean;
}
export interface IDesignerProps extends IDesignerControllerProps {
    droppable?: boolean;
    draggable?: boolean;
    deletable?: boolean;
    cloneable?: boolean;
    resizable?: boolean;
    inlineLayout?: boolean;
    inlineChildrenLayout?: boolean;
    selfRenderChildren?: boolean;
}
export declare type IDesignerControllerPropsMap = Record<string, IDesignerControllerProps>;
export declare type IDesignerPropsMap = Record<string, IDesignerProps>;
export interface IDesignerLocales {
    messages: {
        [ISOCode: string]: {
            [key: string]: string;
        };
    };
    language: string;
}
